//
// Discrete Time CA using rates and parameterised time
//
import java.util.*;
import java.io.*;
import java.awt.*;

public class CARM extends CA {

    double tau;           // the unit or measure of time

    public CARM(int n,int m,double tau,boolean draw){
	super(n,m,draw);
	this.tau = tau;
    }

    public void setDeathRate(int species,double rate){
	deathRate[species] = rate;
	pDeath[species] = 1.0 - Math.pow(Math.E,-rate*tau);
    }

    int numberOfBirths(double lambda){
	double L     = 0.0;
	double U     = gen.nextDouble();
	double factB = 1.0;
	for (int b=0;b<8;b++){
	    if (b > 0) factB = factB * b;
	    L = L + Math.pow(lambda*tau,(double)b) * Math.pow(Math.E,-lambda*tau)/factB;
	    if (L >= U) return b;
	}
	return 8; // size of Moore neighbourhood
    }
	    

    void doBirths(int[][]X,int[][]Y){
	int species = -1;
	for (int i=0;i<n;i++)
	    for (int j=0;j<n;j++){
		species = X[i][j];
		if (species != empty){
		    int numberOfBirths = numberOfBirths(birthRate[species]);
		    for (int k=0;k<numberOfBirths;k++) birth(species,Y,i,j);
		}
	    }
    }
    
    public void doGeneration(){
	copy(A,B);
	doBirths(A,B);
	doDeaths(A,B);
	copy(B,A);
	time = time + tau;
	if (draw){plot(); pause(100);}
	show();
    }

    public static void main(String[] args) {

	int n        = Integer.parseInt(args[0]);          // size of grid
	int maxTime  = Integer.parseInt(args[1]);          // number of iterations
	double tau   = Double.parseDouble(args[2]);        // the click of the clock
	int m        = (args.length/3) - 1;                // n species
	boolean draw = args[args.length-1].equals("draw"); // do we draw?

	int reps = 1;
	if (args[args.length-1].equals("ten")) reps = 10;
	if (args[args.length-1].equals("hundred")) reps = 100;

	while (reps > 0){
	    CARM carm = new CARM(n,m,tau,draw);
	    for (int i=1;i<=m;i++){
		int species = i-1;
		int pop  = Integer.parseInt(args[3*i]);
		carm.setBirthRate(species,Double.parseDouble(args[3*i+1]));
		carm.setDeathRate(species,Double.parseDouble(args[3*i+2]));
		for (int j=0;j<pop;j++) carm.add(species);
	    }
	    carm.show();
	    while (carm.time < maxTime) carm.doGeneration();
	    reps--;
	}
	System.exit(0);
    }
}
//
// ca that use rates, time increment tau and potentially
// multiple births per cell
//
