import java.util.*;
import java.io.*;
import java.awt.*;

public class GCA extends CA {

    RandomSet[] S;             // one for each species
    double[] totalBirthRate;
    double[] totalDeathRate;
    double tau;                // output in increments of tau
    double deltaT;             // used in outputting results

    public GCA(int n,int m,double tau,boolean draw){
	super(n,m,draw);
	this.tau       = tau;
	S              = new RandomSet[m];
	totalBirthRate = new double[m];
	totalDeathRate = new double[m];
	deltaT         = 0.0;
	for (int i=0;i<m;i++) S[i] = new RandomSet(n*n);
    }

    void add(int species) throws RandomSetException {
	int x = freeSpace.delete();
	int i = x/n;
	int j = x%n;
	A[i][j] = species;
	S[species].add(x);
	population[species]++;
	totalPopulation++;
	if (draw) plot(i,j,color[species]);
    }
    //
    // add a new member of species randomly to the population
    //

    void death(int species) throws RandomSetException {
	int x = S[species].delete();
	int i = x/n;
	int j = x%n;
	if (trace){
	    System.out.println("death at ("+ i +","+ j +") ");
	    if (draw){plot(i,j,Color.CYAN); pause(500); unPlot(i,j); pause(500);}
	}
	A[i][j] = empty;
	population[species]--;
	totalPopulation--;
    }
    //
    // kill a member of species from the totalPopulation
    //
    
    void birth(int species) throws RandomSetException {
	int x = S[species].select();
	int i = x/n;
	int j = x%n;
	int point = chooseRandomNeighbour(i,j);
	if (trace){
	    System.out.print("birth from ("+ i +","+ j +") ");
	    System.out.println("onto ("+ point/n +","+ point%n +")");
	    if (draw){plot(point/n,point%n,Color.RED); pause(500); plot(i,j,Color.YELLOW); pause(500);}
	}
	if (A[point/n][point%n] == empty){
	    A[point/n][point%n] = species;
	    S[species].add(point);
	    population[species]++;
	    totalPopulation++;
	}
    }

    static double exp(double lambda){return -Math.log(1 - Math.random()) / lambda;}

    void doGeneration(){
	double lambda = 0.0;
	for (int species=0;species<m;species++){
	    totalBirthRate[species] = birthRate[species] * population[species];
	    totalDeathRate[species] = deathRate[species] * population[species];
	    lambda  = lambda + totalBirthRate[species] + totalDeathRate[species];
	}
	for (int species=0;species<m;species++){
	    pBirth[species] = totalBirthRate[species]/lambda;
	    pDeath[species] = totalDeathRate[species]/lambda;
	}
	deltaT     = deltaT + exp(lambda); // time to next event
	double p   = Math.random();
	double lwb = 0.0;
	for (int species=0;species<m;species++){
	    if (p >= lwb && p < pBirth[species]+lwb){birth(species); break;}
	    lwb = lwb + pBirth[species];
	    if (p >= lwb && p < pDeath[species]+lwb){death(species); break;}
	    lwb = lwb + pDeath[species];
	}
	while (deltaT >= tau){
	    time = time + tau;
	    deltaT = deltaT - tau;
	    if (draw){plot(); pause(100);}
	    show();
	    if (time >= maxTime) break;
	}
    }

    public static void main(String[] args) {
	int n         = Integer.parseInt(args[0]);           // size of grid
	int maxTime   = Integer.parseInt(args[1]);           // number of iterations
	double tau    = Double.parseDouble(args[2]);         // the click of the clock
	int m         = (args.length/3) - 1;                 // n species
	boolean draw  = args[args.length-1].equals("draw");  // do we draw?
	boolean trace = args[args.length-1].equals("trace"); // do we trace and draw?

	int reps = 1;
	if (args[args.length-1].equals("ten")) reps = 10;
	if (args[args.length-1].equals("hundred")) reps = 100;

	while (reps > 0){
	    GCA gca = new GCA(n,m,tau,draw);   
	    for (int i=1;i<=m;i++){
		int species = i-1;
		int popn    = Integer.parseInt(args[3*i]);
		gca.setBirthRate(species,Double.parseDouble(args[3*i+1]));
		gca.setDeathRate(species,Double.parseDouble(args[3*i+2]));
		for (int j=0;j<popn;j++) gca.add(species);
	    }
	    
	    gca.setMaxTime(maxTime);
	    gca.setTrace(trace);
	    gca.show();
	    while (gca.time < maxTime) gca.doGeneration();
	    reps--;
	}
	System.exit(0);
    }
}
